package de.uni_passau.fim.seibt.gitwrapper.repo;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Function;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Collectors;

import de.uni_passau.fim.seibt.gitwrapper.repo.GitWrapper.ExecRes;

/**
 * A git {@link Repository}.
 */
public class Repository {

    private static final Logger LOG = Logger.getLogger(Repository.class.getCanonicalName());

    private GitWrapper git;

    private String url;
    private File dir;

    /**
     * Constructs a new {@link Repository}.
     *
     * @param git
     *         the {@link GitWrapper} to use
     * @param url
     *         the url the {@link Repository} was cloned from
     * @param dir
     *         the directory containing the {@link Repository}
     */
    Repository(GitWrapper git, String url, File dir) {
        this.git = git;
        this.url = url;

        try {
            this.dir = dir.getCanonicalFile();
        } catch (IOException e) {
            LOG.log(Level.WARNING, e, () -> "Exception computing the canonical file for " + dir + ". Using the absolute file instead.");
            this.dir = dir.getAbsoluteFile();
        }
    }

    /**
     * Returns the merge commits in this repository.
     *
     * @return the merge commits
     */
    public List<Commit> getMergeCommits() {
        Optional<ExecRes> revList = git.exec(dir, "rev-list", "--all", "--merges");
        Function<ExecRes, List<Commit>> toCommitList = res -> {
            String[] lines = res.output.split("[\\r?\\n]+");

            LOG.fine(() -> String.format("Found %d merge commits in %s.", lines.length, this));

            return Arrays.stream(lines).map(id -> new Commit(this, id)).collect(Collectors.toList());
        };

        return revList.map(toCommitList).orElse(Collections.emptyList());
    }

    public GitWrapper getGit() {
        return git;
    }

    public String getUrl() {
        return url;
    }

    public File getDir() {
        return dir;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }

        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        Repository that = (Repository) o;
        return Objects.equals(url, that.url) && Objects.equals(dir, that.dir);
    }

    @Override
    public int hashCode() {
        return Objects.hash(url, dir);
    }

    @Override
    public String toString() {
        return String.format("Repository{url=%s, dir=%s}", url, dir);
    }
}
